@@ -3,7 +3,7 @@ package Promise::ES6;
 use strict;
 use warnings;
 
-our $VERSION = '0.22_04';
+our $VERSION = '0.23';
 
 =encoding utf-8
 
@@ -68,6 +68,9 @@ not a list.
 =item * Unhandled rejections are reported via C<warn()>. (See below
 for details.)
 
+=item * Undefined or empty rejection values trigger a warning.
+This provides the same value as Perl’s own warning on C<die(undef)>.
+
 =item * The L<Promises/A+ test suite|https://github.com/promises-aplus/promises-tests> avoids testing the case where an “executor”
 function’s resolve callback itself receives another promise, e.g.:
 
@@ -104,8 +107,7 @@ In JavaScript, the following …
     console.log(2);
 
 … will log C<2> then C<1> because JavaScript’s C<then()> defers execution
-of its callbacks until the end of the current iteration through JavaScript’s
-event loop.
+of its callbacks until between iterations through JavaScript’s event loop.
 
 Perl, of course, has no built-in event loop. This module accommodates that by
 implementing B<synchronous> promises by default rather than asynchronous ones.
@@ -139,10 +141,10 @@ call stack limits. For example, the following (admittedly contrived) code:
 
 … will eventually fail because it will reach Perl’s call stack size limit.
 
-That problem probably won’t matter in most applications. If you want to
-avoid it, though, you’ll need asynchronous promises.
+That problem probably won’t affect most applications. The best way to
+avoid it, though, is to use asynchronous promises, à la JavaScript.
 
-To do that, first, choose one of the following event interfaces:
+To do that, first choose one of the following event interfaces:
 
 =over
 
@@ -169,6 +171,11 @@ Then, before you start creating promises, do this:
 That’s it! Promise::ES6 instances will now work asynchronously rather than
 synchronously.
 
+Note that this changes Promise::ES6 I<globally>. In IO::Async’s case, it
+won’t increase the passed-in L<IO::Async::Loop> instance’s reference count,
+but if that loop object goes away, Promise::ES6 won’t work until you call
+C<use_event()> again.
+
 B<IMPORTANT:> For the best long-term scalability and flexibility,
 your code should work with either synchronous or asynchronous promises.
 
@@ -195,7 +202,7 @@ to be canceled. See L<Net::Curl::Promiser> for an example of this approach.
 You’ll need to decide if it makes more sense for your application to leave
 a canceled query in the “pending” state or to “settle” (i.e., resolve or
 reject) it. All things being equal, I feel the first approach is the most
-intuitive.
+intuitive, while the latter ends up being “cleaner”.
 
 =head1 MEMORY LEAKS
 
@@ -258,8 +265,8 @@ If you’re not sure of what promises are, there are several good
 introductions to the topic. You might start with
 L<this one|https://developer.mozilla.org/en-US/docs/Web/JavaScript/Guide/Using_promises>.
 
-L<Promise::XS> is a lot like this library but implemented mostly in XS for
-speed. I refactored it from L<AnyEvent::XSPromises>.
+L<Promise::XS> is my refactor of L<AnyEvent::XSPromises>. It’s a lot like
+this library but implemented mostly in XS for speed.
 
 L<Promises> is another pure-Perl Promise implementation.
 
@@ -292,8 +299,12 @@ sub use_event {
     my $modname = $name;
     $modname =~ tr<:><>d;
 
+    my @saved_errs = ($!, $@);
+
     require "Promise/ES6/Event/$modname.pm";
 
+    ($!, $@) = @saved_errs;
+
     $_EVENT = $name;
 
     # We need to block redefinition and (for AnyEvent)
@@ -313,9 +324,9 @@ sub resolve {
 }
 
 sub reject {
-    my ( $class, $reason ) = @_;
+    my ( $class, @reason ) = @_;
 
-    $class->new( sub { $_[1]->($reason) } );
+    $class->new( sub { $_[1]->(@reason) } );
 }
 
 sub all {
@@ -412,17 +423,26 @@ sub race {
     return $new;
 }
 
+sub _aS_fulfilled {
+    return { status => 'fulfilled', value => $_[0] };
+}
+
+sub _aS_rejected {
+    return { status => 'rejected', reason => $_[0] };
+}
+
+sub _aS_map {
+    return $_->then( \&_aS_fulfilled, \&_aS_rejected );
+}
+
 sub allSettled {
     my ( $class, $iterable ) = @_;
 
     my @promises = map { UNIVERSAL::can( $_, 'then' ) ? $_ : $class->resolve($_) } @$iterable;
 
-    return $class->all( [ map {
-        $_->then(
-            sub { { status => 'fulfilled', value => $_[0] } },
-            sub { { status => 'rejected', reason => $_[0] } },
-        );
-    } @promises ] )
+    @promises = map( _aS_map, @promises );
+
+    return $class->all( \@promises );
 }
 
 #----------------------------------------------------------------------
